# Directory Vars
ARTIFACT_FOLDER = "Artifacts"

# Applications vars
APPLICATIONS_FILE = "applications.cache"
APPLICATION_FILE = ".cache"
APPLICATION_VERSIONS_FILE = ".versions.cache"
APPLICATION_VERSION_FILE = ".version.cache"
APPLICATION_FOLDER = 'application_data'

# Modules vars
MODULES_FILE = "modules.cache"
MODULES_FOLDER = 'module_data'
MODULE_FILE = ".cache"
MODULE_VERSIONS_FILE = ".versions.cache"
MODULE_VERSION_FILE = ".version.cache"

# Environments vars
ENVIRONMENTS_FILE = "environments.cache"
ENVIRONMENT_FILE = ".cache"
ENVIRONMENT_APPS_FILE = ".applications.cache"
ENVIRONMENT_APPLICATION_FILE = ".applications.cache"
ENVIRONMENT_VERSIONS_FILE = ".versions.cache"
ENVIRONMENT_FOLDER = 'environment_data'

# Environments vars
DEPLOYMENTS_FILE = "deployments.cache"
DEPLOYMENT_FILE = ".cache"
DEPLOYMENT_PLAN_FILE = ".plan.cache"
DEPLOYMENT_MANIFEST_FILE = "deployment_manifest.cache"
DEPLOYMENT_STATUS_FILE = ".status.cache"
DEPLOYMENT_FOLDER = "deployment_data"

# CICD Probe vars
PROBE_APPLICATION_SCAN_FILE = ".probe.cache"
PROBE_FOLDER = "cicd_probe_data"

# BDD Framework vars
BDD_FRAMEWORK_TEST_RUN_FILE = ".testrun.cache"
BDD_FRAMEWORK_TEST_ENDPOINTS_FILE = "test.endpoints.cache"
BDD_FRAMEWORK_FOLDER = "bdd_data"

# JUNIT vars
JUNIT_TEST_RESULTS_FILE = "junit-result.xml"