# Base CICD Probe Variables
BDD_HTTP_PROTO = "https"
BDD_API_ENDPOINT = "BDDFramework/rest"
BDD_API_VERSION = 1

# Test Runner Endpoint Variables
BDD_TEST_RUNNER_ENDPOINT = "BDDTestRunner"
BDD_RUNNER_SUCCESS_CODE = 200