# Base CICD Probe Variables
PROBE_HTTP_PROTO = "https"
PROBE_API_ENDPOINT = "CI_CDProbe/rest"
PROBE_API_VERSION = 1

# Scan Endpoints
SCAN_BDD_TESTS_ENDPOINT = "ScanBDDTestEndpoints"
PROBE_SCAN_SUCCESS_CODE = 200